#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: VOID Onset(ibak,sper,tm_indx,				*
 * 			tc,amp_indx,pt0,pt1,pt2,pt3,pt4)  		*
 *					-Calculate onset parameters	*
 *----------------------------------------------------------------------*
 *	Arguments:							*
 *			LONG ibak    - # of P-T values looked back	*
 *			LONG sper    - period of signal (samples)	*
 *			LONG tm_indx - index to reference time of	*
 *					 first P-T value of signal	*
 *			LONG tc      - time correction			*
 *			LONG amp_indx- index to first P-T value of	*
 *					  the signal			*
 *			LONG pt0     - value 1 for SNR calc		*
 *			LONG pt1     - value 2 for SNR calc		*
 *			LONG pt2     - value 3 for SNR calc		*
 *			LONG pt3     - value 4 for SNR calc		*
 *			LONG pt4     - value 5 for SNR calc		*
 *----------------------------------------------------------------------*
 *	Returns:	Nothing (VOID)					*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	Calculates period, amplitude,SNR series (O-F 83-785), and 	*
 *	converts sample number to time. Outputs the estimates.		*
 *----------------------------------------------------------------------*/

VOID Onset(ibak, sper, tm_indx, tc, amp_indx, pt0,
	pt1, pt2, pt3, pt4)
LONG ibak, sper, tm_indx, amp_indx;
LONG pt0, pt1, pt2, pt3, pt4, tc;
{

	LONG i, indx2, j, isnr[5];
	LONG mx_amp, base, pt[5], temp, onset1;
	LONG t_samp, d_samp;
	TEXT ipol;

	struct s_onset  onsetdata;

/*---------------get polarity of first break of the signal--------------*/

	if (pt2 > 0)
	    ipol = 'd';		       /* pt2 is first P-T of signal */
	else
	    ipol = 'c';

/*-----------------------load array for SNR calculation------------------*/
/*------------------------see OF Report 83-785---------------------------*/
	pt[0] = pt0;		       /* 2nd P-T before signal */
	pt[1] = pt1;		       /* P-T immediately before signal */
	pt[2] = pt2;		       /* 1st P-T of signal */
	pt[3] = pt3;		       /* 2nd P-T of signal */
	pt[4] = pt4;		       /* 3rd P-T of signal */

/*------------------do the SNR calculation without using long division----*/

	for (j = 0; j < 5; j++) {
	    base = con_ptr->twosd - (con_ptr->twosd >> 1);
	    for (i = 0; i < 10; i++) {
	        isnr[j] = i;
	        if (absval(pt[j]) < base)
	    	break;
	        base += con_ptr->twosd;
	    }
	}

/*------------------get the maximum amplitude of the first 4 cycles-----*/

	indx2 = amp_indx;
	mx_amp = 0;
	for (j = 0; j < 8; j++) {
	    temp = absval(con_ptr->buf_amp[indx2]);
	    if (temp > mx_amp)
	        mx_amp = temp;
	    if (indx2++ == B_M1)
	        indx2 = 0;		       /* indx2 incremented */
	}

/*------------calculate the Period(in seconds) of the first 4 cycles----*/

	Period(sper);

/*--------Compute which record the data came from so that the time will------*/
/*--------be calculated correctly.  -----------------------------------------*/

	t_samp = con_ptr->buf_tim[tm_indx];
	d_samp = con_ptr->cur_rec - con_ptr->buf_rec[tm_indx];
	if (d_samp < 0)
	    d_samp += CUR_MAX;

	if (d_samp > 0)
	    t_samp -= con_ptr->sam_ch * d_samp;

/*-----------------set up the onset time for output--------------------------*/

	onset1 = Time_f(t_samp, tc);

	while (onset1<0)     {  onset1+=60;      etime.min--;    }
	while (onset1>59)    {  onset1-=60;      etime.min++;    }

	while (etime.min<0)  {	etime.min+=60;   etime.hr--;	}
	while (etime.min>59) {	etime.min-=60;   etime.hr++;	}
	while (etime.hr<0)   {	etime.hr+=24;    etime.day--;	}
	while (etime.hr>23)  {	etime.hr-=24;    etime.day++;	}

	while (etime.day>etime.day_yr) {
		etime.day -= etime.day_yr-1;
		etime.yr++;
	}

	while (etime.day<1) {
		etime.yr--;
		etime.day += etime.prv_yr;	/* Fix 3-Feb-89/SH */
	}

	if (con_ptr->ponset) {
	    printf(" %s", con_ptr->ch_name);
	    printf(" %c %ld ", ipol, ibak);
	    for (i = 0; i < 5; i++)
		printf("%ld", isnr[i]);
	    printf(" %ld ", etime.yr);
	    printf("%3ld ", etime.day);
	    printf("%02ld", etime.hr);
	    printf(":%02ld:", etime.min);
	    printf("%02ld.%03ld %7ld %2ld.%03ld %7ld",
	    	onset1, t_rval, mx_amp, p_lval, p_rval, con_ptr->twosd);
/*		Was th1 exceeded in the detection? A=yes, B=no.	*/
	    printf(" %c ", con_ptr->iset ? 'A' : 'B');
	    printf("\n");
	}

/*--This code is for writing data into headers of China System records --*/
	onsetdata.new_onset = TRUE;	       /* New onset has occured */
	onsetdata.o_polar = ipol;
	onsetdata.o_dalgo = con_ptr->iset ? 'A' : 'B';
	for (i = 0; i < 5; i++)
	    onsetdata.o_snr[i] = isnr[i];
	onsetdata.o_year = etime.yr;
	onsetdata.o_days = etime.day;
	onsetdata.o_hours = etime.hr;
	onsetdata.o_mins = etime.min;
	onsetdata.o_secs = onset1;
	onsetdata.o_msecs = t_rval;
	onsetdata.o_amps = mx_amp;
	onsetdata.o_pl = p_lval;
	onsetdata.o_pr = p_rval;
	onsetdata.o_large = con_ptr->twosd;

	/* Optional processing routine for onsets */
	if (con_ptr->onsproc != NULL)
	    (*(con_ptr->onsproc))(&onsetdata);/* send onsproc the address of
						the output list */

/*--------etime will be changed when events are detected - backup--------*/

	etime.day_yr = htime.day_yr;
	etime.prv_yr = htime.prv_yr;
	etime.yr = htime.yr;
	etime.day = htime.day;
	etime.hr = htime.hr;
	etime.min = htime.min;
	etime.sec = htime.sec;
	etime.msec = htime.msec;

/*----The thresholds are raised to permit processing the coda of events, --*/
/*     also, the indices kk and index are set so that Ptwo is called     */
/*     expeditiously when processing of the signal coda begins.          */

/*---There is a provision for tuning to process local events.  Here --------*/
/*    the maximum P-T (first four cycles) of the signal is loaded into	*/
/*    the thresholds and into the tsstak of P_two.			*/

							/* Note 1 */
#ifdef LOCAL_EV
	con_ptr->twosd = mx_amp;
	for (i = 0; i < con_ptr->val_avg; i++)
	    con_ptr->tsstak[i] = mx_amp;
#else
#define bump(x) ((x) + ((x)>>2)) /*bump value by a factor of 1.25 */
	for(i = 0; i < 16; i++)
		con_ptr->tsstak[i] = bump(con_ptr->tsstak[i]);
	con_ptr->twosd = bump(con_ptr->twosd);
#endif

/*------------Raise the thresholds here by calling xth------------------*/
/*		after raising twosd (above)				*/

	con_ptr->th1 = Xth(con_ptr->xth1);
	con_ptr->th2 = Xth(con_ptr->xth2);
	con_ptr->th3 = Xth(con_ptr->xth3);
	con_ptr->thx = Xth(con_ptr->xthx);

/*------------To synchronize on-line and off-line detectors-------------*/
/*			See P_one for definitions			*/
						/* Note 2 */

	con_ptr->maxamp = 0;
	con_ptr->s_sum_sc = 0;
	con_ptr->sum_s_c = 0;
	con_ptr->prev_slope = 0;
	con_ptr->max_y = con_ptr->last_y = 0;
	con_ptr->tim_of_max = con_ptr->last_x = 0;
	con_ptr->index = 19;	       /*  ensure call to Ptwo expeditiously */

	con_ptr->kk = con_ptr->val_avg - 1;
	    			       /* ensure new average expeditiously */

	return;
}

#ifdef JNMCOMMENT
/******************************************************************************

				    NOTES

	[1]  The thresholds should be raised so that the coda of events can
	     be processed.  Indeed, if the thresholds were not raised, and 
	     if the noise field changed dramatically (as happens at some
	     stations), the detector could not track the background and would
	     continuously produce detections.  Provision is made for two
	     different modes of processing the event coda.  If one is
	     primarily interested in local events, one might use the code
	     included when LOCAL_EV is defined.  Here one might input a
	     small number for val_avg (see P_two) so that the detector 
	     could track changes in the code more efficiently.  Alternatively,
	     if one is processing data of teleseisms, and one wishes to 
	     record much of the coda, the default option, with large values
	     (8 - 16) for val_avg, should be employed.  In this instance, for
	     large events, one can reasonably expect several retriggers in
	     the coda, some of which might be caused merely by the elevated
	     level of overall activity, rather than by distinct phases.

	[2]  In implementing the code to run on a new system, one often
	     wishes to compare the output on the system being developed with
	     a benchmark that is run on a different system.  The variables
	     are set to facilitate such comparisons.

****************************************************************************

			EXPLANATION OF VARIABLES

	amp_indx-- an index to the first P-T value of the signal.
		    Set in routine Onsetq.

	base	-- a reference value used to calculate the quality
		    numbers of the onset of the signal.

	B_M1	-- the maximum index of the circular buffers of size E_B.

	buf_amp -- a buffer of size E_B (maximum index B_M1) that
		    contains the signed P-T values.  Loaded in 
  		    routine Event.

	buf_tim -- a buffer that contains the number of samples
		    from a reference time for each P-T value in 
		    buf_amp.  Same size as buf_amp.

	buf_rec -- contains the (arbitrary) record number of each P-T value.
		    Same size as buf_amp.

	CUR_MAX	-- MAX value for record identification.  Value 
		    allows a relative age of record to be determined.
		    When the age reaches CUR_MAX, the record number is
        	    reset.

	cur_rec -- the (arbitrary) current record number.  Incremented
		    in E_detect, saved in P_one, loaded into buf_rec
		    in Event.

	d_samp	-- Delta samples - age of data in reference to the 
		    current record.  Used to compute times accurately.

	etime.xxx-- the time that was initially read in the headers
		    of the records.  These times may be changed in
		    the routine Ck_t_kont.

	etime.prv_yr-- the number of days in the previous year.
		    
	htime.xxx-- Since the time calculation routines modify
		    etime for the onset time, we must backup etime
		    and restore it so that the next event in the
		    same record can be correctly calculated.

	ibak	-- the number of P-T values that Onsetq looked
		    back to find the first P-T value of the signal.
		    (It is the number of P-T values before fst_flg.)

	indx2	-- an index that references values in the circular
		    buffers that were set up in routine Event.

	ipol	-- the polarity of the onset of the signal (c or d).

	iset	-- TRUE if P-T >= th1, FALSE otherwise. See Event.c.

	isnr[]	-- the quality numbers (signal-to-noise ratios) that
		    that are integer values each of which is in the
		    interval 0 <= isnr <= 9.  Dimensionless.

	mx_amp	-- maximum amplitude of the first four cycles (first
		    8 P-T values) of the signal. Counts.

	new_onset -- a flag set=TRUE if a signal was declared in the
		      current record.  Set=FALSE otherwise.

	onset1	-- the integer number of seconds from the reference
		    header time.  The remainder of the seconds is
		    in t_rval.

	p_lval	-- the integer part of the signal period.  Set in
		    routine Period.  Seconds.

	p_rval	-- the remainder of the signal period.  Set in
		    routine Period.  Milliseconds.

	pt[]	-- this array contains the values of the
		    first P-T amplitude of the signal (pt[2]), plus
		    the 2 P-T amplitudes on either side of it.

	sam_ch	-- an input value, samples per channel of the record.

	sper	-- the period of the signal in samples.  Set in
		    routine Onsetq.

	tc	-- the time correction that is applied to the reference
		    time.  Set in routine Onsetq. Milliseconds.


	tm_indx -- index to the reference time of the signal.  Set in
		    routine Onsetq.

	t_rval	-- the fraction part of onset.  This value is set 
		    in routine Time_f.  Milliseconds.

	t_samp	-- the reference time (in samples) for the onset of
		    the signal.  By an intermediate process, the
		    value is initialized at the beginning of each
		    record in E_detect.

	twosd	-- an estimate of twice the sample standard deviation
		    of the P-T values.  Set in routine P_two. Counts.

**************************************************************************/
#endif
