#include <detect.h>
#define RO 2 /* round off used for period estimated from 8 P-T values */

/*----------------------------------------------------------------------*
 *	Function: VOID Onsetq() - Estimate onset			*
 *----------------------------------------------------------------------*
 *	Arguments:	None						*
 *----------------------------------------------------------------------*
 *	Returns:	Nothing (VOID)					*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	As one might remember,the buffers for each candidate signal 	*
 *	 have at least 4 P-T values before the first one that was	*
 *	 >= th2.  Onsetq compares the last two of these four with yet	*
 *	 another threshold, th3 (th3 < th2).  In addition, a test is	*
 *	 is performed to see whether or not the P-T value looks like	*
 *	 it is part of the signal.  Here "looks like" is determined	*
 *	 by the period of the signal.  These tests are to search	*	
 *	 for a signal onset that is smaller than th2.  When the first	*
 *	 P-T value of the signal is found, it is flagged.  In P_one,	*
 *	 recall the reference time of each P-T value is given when the 	*
 *	 P-T value is declared; hence the time is for the "trailing	*
 *	 edge" of the P-T value.  Therefore the signal onset occurs	*
 *	 before the time of the first P-T value of the signal.		*
 *	 The algorithm considers two possibilities for the onset:	*
 *	 It is either the time of the P-T value that immediately pre-	*
 *	 ceeds the signal, or if this P-T value occurs too far ahead	*
 *	 of the first P-T value of the signal, a correction is applied	*
 *	 to the time of this P-T value. Here "too far" is deter-	*
 *	 mined by the measured period of the signal.  The correction	*
 *	 (0 or 500 ms for the SP and BB) and an index to the reference	*
 *	 P-T are sent to routine Onset for conversion to Universal Time.*
 *	 								*
 *	In addition to this index and other useful information, Onsetq	*
 *	 sends Onset the amplitudes of the two P-T values that occur	*
 *	 on either side of the first P-T value of the signal.  These 	*
 *	 five P-T values (the two prior to the signal and the first	*
 *	 three of the signal) are used by Onset to estimate the		*
 *	 quality of the time determination of the beginning of the	*
 *	 signal (see O-F Report 83-785).				*
 *----------------------------------------------------------------------*/

VOID Onsetq() {

	WORD flg_p1, flg_p2, flg_m1, flg_m2, flg_m3, flg_m4, lb,
	    i1, i2, i3, i4, i5, i6, per_bnd, per_sc, kase, per_sav;
	LONG tc;

	kase = 0;

/*--------Setting up indices that are corrected for a circular buffer --*/

	flg_m4 = con_ptr->fst_flg - 4;   /* 4 P-T values before the first 
					  flagged one */
	if (flg_m4 < 0)
	    flg_m4 = con_ptr->fst_flg + B_M4;	/* etc */

	flg_m3 = con_ptr->fst_flg - 3;
	if (flg_m3 < 0)
	    flg_m3 = con_ptr->fst_flg + B_M3;

	flg_m2 = con_ptr->fst_flg - 2;
	if (flg_m2 < 0)
	    flg_m2 = con_ptr->fst_flg + B_M2;

	flg_m1 = con_ptr->fst_flg - 1;
	if (flg_m1 < 0)
	    flg_m1 = con_ptr->fst_flg + B_M1;

	flg_p1 = con_ptr->fst_flg + 1;   /* 1 P-T after the first flagged one */
	if (flg_p1 >= E_B)
	    flg_p1 = con_ptr->fst_flg - B_M1;

	flg_p2 = con_ptr->fst_flg + 2;   /* etc */
	if (flg_p2 >= E_B)
	    flg_p2 = con_ptr->fst_flg - B_M2;

	per_bnd = con_ptr->fst_flg + 8;  /* index of the last P-T used to
					  estimate the period */
	if (per_bnd > B_M1)
	    per_bnd -= E_B;

	per_sc = con_ptr->buf_sc[per_bnd] - 
		con_ptr->buf_sc[con_ptr->fst_flg] + RO;
	per_sc >>= 2;
	per_sav = per_sc;	 /* per_sc may be changed below, in the
					  process that estimates the onset of
					  the signal */

	if (per_sc < con_ptr->sam_sec)
	    per_sc = con_ptr->sam_sec;   /* 1 sec */
	con_ptr->haf_per = per_sc >> 1;

/*----------------Code for looking back 2 p-t values---------------------*/
					/* Note 1 */

	if ((con_ptr->buf_sc[flg_m1] - con_ptr->buf_sc[flg_m2]) <= per_sc)
	    if (absval (con_ptr->buf_amp[flg_m2]) >= con_ptr->th3)
		kase = ((con_ptr->buf_sc[flg_m2] -
			con_ptr->buf_sc[flg_m3])
		    > con_ptr->haf_per) ? 1 : 2;

/*----------------Code for looking back 1 p-t value---------------------*/
					/* Note 2 */

	if ((kase == 0) && (con_ptr->buf_sc[con_ptr->fst_flg] -
		con_ptr->buf_sc[flg_m1]) <= per_sc)
	    if (absval(con_ptr->buf_amp[flg_m1]) >= con_ptr->th3)
		kase = ((con_ptr->buf_sc[flg_m1] -
			con_ptr->buf_sc[flg_m2])
		    > con_ptr->haf_per) ? 3 : 4;

/*----------------Code for looking back 0 p-t values--------------------*/

	if ((kase == 0) && (con_ptr->buf_sc[con_ptr->fst_flg] -
		con_ptr->buf_sc[flg_m1]) <= con_ptr->haf_per)
	    kase = 5;

	switch (kase) {

	    case 1: 
		lb = 2;
		tc = con_ptr->def_tc;

		i1 = flg_m2;	       /* index to reference time */
		i2 = flg_m4;	       /* index to amplitude  of P-T */
		i3 = flg_m3;
		i4 = flg_m2;
		i5 = flg_m1;
		i6 = con_ptr->fst_flg;   /*  "  "   "  "  " */

		break;
	    case 2: 
		lb = 2;
		tc = 0;

		i1 = flg_m3;	       /* index to reference time */
		i2 = flg_m4;	       /* index to amplitude  of P-T */
		i3 = flg_m3;
		i4 = flg_m2;
		i5 = flg_m1;
		i6 = con_ptr->fst_flg;  /* "     "   "   " */

		break;
	    case 3: 
		lb = 1;
		tc = con_ptr->def_tc;

		i1 = flg_m1;	       /* index to reference time */
		i2 = flg_m3;	       /* index to amplitude  of P-T */
		i3 = flg_m2;
		i4 = flg_m1;
		i5 = con_ptr->fst_flg;
		i6 = flg_p1;	       /*  "  "   "  "  " */

		break;
	    case 4: 
		lb = 1;
		tc = 0;

		i1 = flg_m2;	       /* index to reference time */
		i2 = flg_m3;	       /* index to amplitude  of P-T */
		i3 = flg_m2;
		i4 = flg_m1;
		i5 = con_ptr->fst_flg;
		i6 = flg_p1;	       /*  "  "   "  "  " */

		break;
	    case 5: 
		lb = 0;
		tc = 0;

		i1 = flg_m1;	       /* index to reference time */
		i2 = flg_m2;	       /* index to amplitude  of P-T */
		i3 = flg_m1;
		i4 = con_ptr->fst_flg;
		i5 = flg_p1;
		i6 = flg_p2;	       /*  "  "   "  "  " */

		break;
	    default: 
		lb = 0;
		tc = con_ptr->def_tc;

		i1 = con_ptr->fst_flg;   /* index to reference time */
		i2 = flg_m2;	       /* index to amplitude  of P-T */
		i3 = flg_m1;
		i4 = con_ptr->fst_flg;
		i5 = flg_p1;
		i6 = flg_p2;	       /*  "  "   "  "  " */

		break;

	}

	/* printf("Case=%d, tc=%d, per_sc=%d,haf_per=%d\n",kase,tc,per_sc,
		con_ptr->haf_per); */

	/* printf("m1=%d,m2=%d,m3=%d,m4=%d\n",
		con_ptr->buf_sc[flg_m1],
		con_ptr->buf_sc[flg_m2],
		con_ptr->buf_sc[flg_m3],
		con_ptr->buf_sc[flg_m4]); */

	Onset(lb, per_sav, i1, tc, i4, con_ptr->buf_amp[i2],
		con_ptr->buf_amp[i3], con_ptr->buf_amp[i4],
		con_ptr->buf_amp[i5], con_ptr->buf_amp[i6]);

	return;
}

#ifdef JNMCOMMENT
/****************************************************************************

				NOTES

	[1]	Algorithm for looking back two P-T values.  First we
		 check to see that the difference between the two P-T
		 values (fst_flg-1 and fst_flg-2) is not too large
		 (1 sec or the period, whichever is greater).  If the
		 P-T values pass this test, we then check whether or
		 not fst_flg-2 >= th3.  If either of these two tests 
		 fail, we proceed to the algorithm for looking back
		 one P-T value.  However, if both tests succeed, we then
		 test to see what time we will use for the onset; if the
		 preceeding (fst_flg-3) P-T value occurs too far ahead
		 of fst_flg-2, we use the time of fst_flg-2 - 0.5 sec (an
		 input value), otherwise we use the time of fst_flg-3.
		 Here "too far" is per_sc/2.  (This contrasts with the
		 FORTRAN progam which used a constant of 10 (sam_sec/2)
		 instead of per_sc/2. For general purpose application,
		 especially for the Long Period System, and perhaps
		 for the Broad Band as well, the 0.5 sec (10 samples)
		 appears too small.)


	[2]	Algorithm for looking back one P-T value.  The logic
		 parallels that explained in [1] above.  Note that 
		 kase != 0 if both time and amplitude criteria were met
		 in [1] above.


******************************************************************************

			EXPLANATION OF VARIABLES


	B_M1
	B_M2	-- the size (E_B) of the large P-T buffers minus 1,...,minus 4
	B_M3
	B_M4

	buf_amp -- a buffer that contains the signed P-T amplitudes.

	buf_rec -- a buffer that contains the record numbers in which
		    each of the P-T values of buf_amp occurred.

	buf_sc	-- a buffer that indicates time by using the sum of
		    the samples.  This clock is initialized when
		    the abbreviated buffers are downloaded in Event
		    (when an event is judged possibly to be in progress).

	def_tc	-- an input value that is loaded into tc (for SP and BB of
		    the China System it is 500 (ms) ).

	fst_flg	-- an index to the first P-T value of the current signal
		    that is >= th2.

	flg_m1	-- fst_flg minus 1	
	flg_m2  -- fst_flg minus 2
	flg_m3	-- fst_flg minus 3
	flg_m4	-- fst_flg minus 4
	flg_p1	-- fst_flg plus 1
	flg_p2	-- fst_flg plus 2

	i1	
	i2	
	i3	-- indices that are used to reference values in the 
	i4	    large buffers of Event (buf_amp, buf_rec, buf_tim).
	i5	    The index i1 references time, the others, amplitude.
	i6

	kase   	-- an index that is used in the lookback proceedure.

	lb	-- the number of P-T values that Onsetq looked back to
		    determine the first P-T of the signal (0, 1, or 2).

	per_bnd	-- a bound that is used to estimate the period of the
		    first four cycles of the signal.

	per_sav	-- saved value of per_sc, used for outputting the period
		    of the signal. Samples.

	per_sc	-- the period of the signal (samples).

	RO	-- round off for estimating the period of the signal. Samples.

	sam_sec -- digitizing rate of the current seismic data record.
		    Samples per second.

	haf_per	-- samples per one-half period.

	tc	-- a time correction that is applied (see discussion in
		    the header of this routine).  Milliseconds.

**************************************************************************/
#endif
