#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: BOOL E_steimload(detector,indata,offset) - load harv	*
 *----------------------------------------------------------------------*
 *	Arguments:	struct detect_info *detector - calling info	*
 *			UBYTE *indata - where the data is		*
 *			WORD offset - what multiplexed channel		*
 *----------------------------------------------------------------------*
 *	Returns:	TRUE - Data valid and converted			*
 *			FALSE - the recordtype did not match		*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	Examine a HARVARD/IRIS-1 Steim compresses record in its		*
 *	internal format.  Determine the record type and compare the	*
 *	record type with the selection critera in the structure		*
 *	"detector".  If the record types are the same, decode the	*
 *	rest of the data and return TRUE.  Otherwise, do no more, and	*
 *	return FALSE.							*
 *----------------------------------------------------------------------*/

/*----Structures to describe Harvard format with Steim Compression Frames----*/

struct Header_Frame {
	ULONG	ctlflags;
	BYTE	frame_type;
	BYTE	component;
	BYTE	stream;
	BYTE	resync;
	UBYTE	station[4];
	WORD	millisec;
	WORD	time_mark;
	LONG	samp_1;
	LONG	samp_2;
	BYTE	rate;
	BYTE	mass_pos;
	struct {
		BYTE	year;
		BYTE	month;
		BYTE	day;
		BYTE	hour;
		BYTE	mins;
		BYTE	sec;
	} rec_time;
} *recordheader;

struct Compression_Frame {
	struct {
		ULONG	ctlflags;
		union {
			struct { BYTE byteval[4]; } type_1;
			struct { WORD wordval[2]; } type_2;
			struct { LONG longval;    } type_3;
		} chunkstore[15];
	} framestore[16];
} *steimframe;

/*--------Extract the control nibble from the frame control longword--------*/

CtlWordType(inlong,pos)
ULONG inlong;
int pos;
{

	int sf,out;

	sf = (15-(pos+1))*2;
	if (sf>0) inlong >>= sf;

	out = inlong&0x03;
	return(out);
}
		
#define LocGM68_WORD(a) a
#define LocGM68_LONG(a) a

UBYTE montab[] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

BOOL E_steimload(detector, indata, offset)
struct detect_info *detector;
UBYTE * indata;
WORD offset;
{

	LONG a,b,c,d;
	UBYTE *ci,cr;
	int lastmode;
	WORD pday,leap;

	WORD frame,chunk,oid,frtype;
	LONG l_data,*oarray,tdata;
	BOOL fskip;

	if (indata==NULL) return;

	recordheader = (struct Header_Frame *) indata;
	steimframe = (struct Compression_Frame *) indata;

	/* Make a composite detector type from component and stream */
	a = (recordheader->component + (recordheader->stream*20)) + 1;

	if (detector->drectyp != a)
	    return(FALSE);		       /* Not this one */

	detector->samrte = 0;
	a = recordheader->rate;
	if (a!=0) {
		if (a>0) detector->samrte = a*100;	
		else {
			b = 100/(-a);
			detector->samrte = b;
		}
	}

/*-------------------Compute a complete date-----------------------------*/

	pday = 0;
	detector->startt.hr = recordheader->rec_time.hour;
	detector->startt.min = recordheader->rec_time.mins;
	detector->startt.sec = recordheader->rec_time.sec;
	detector->startt.msec = LocGM68_WORD(recordheader->millisec);

	a = recordheader->rate;
	if (a<0)	b = 1000*(-a);
	else            b = 1000/a;
	
	b *= LocGM68_WORD(recordheader->time_mark)-1;

	d = detector->startt.msec - b;
	c = detector->startt.sec;
	b = detector->startt.min;
	a = detector->startt.hr;

	/* Normalize the date *&/

	while (d<0) {
		d += 1000;
		c --;
	}

	while (c<0) {
		c += 60;
		b --;
	}

	while (b<0) {
		b += 60;
		a --;
	}

	while (a<0) {
		b+= 24;
		pday --;
	}

	while (d>=1000) {
		d -= 1000;
		c ++;
	}

	while (c>=60) {
		c -= 60;
		b ++;
	}

	while (b>=60) {
		b -= 60;
		a++;
	}

	while (a>=24) {
		a -= 24;
		pday ++;
	}

	detector->startt.msec = d;
	detector->startt.sec = c;
	detector->startt.min = b;
	detector->startt.hr = a;

	a = recordheader->rec_time.year + 1900;

	if (pday<0&&
	    recordheader->rec_time.month==1&&
	    (recordheader->rec_time.day+pday<1)) 
			a --;			/* It's in prev year */

	if (pday>0&&
	    recordheader->rec_time.month==12&&
	    (recordheader->rec_time.day+pday>31)) 
			a ++;			/* It's in next year */

	if (a != detector->startt.yr) {
	    leap = 0;
	    detector->startt.yr = a;
	    if (!(a % 4))
	        leap = 1;
	    if (!(a % 100))
	        leap = 0;
	    if (!(a % 400))
	        leap = 1;
	    detector->startt.day_yr = 365 + leap;
	    montab[1] = 29-leap;
	    a = a --;
	    if (!(a % 4))
	        leap = 1;
	    if (!(a % 100))
	        leap = 0;
	    if (!(a % 400))
	        leap = 1;
	    detector->startt.prv_yr = 365 + leap;
	}

	detector->startt.day = pday + recordheader->rec_time.day;
	for (a=0; a<recordheader->rec_time.month-1; a++)
		detector->startt.day += montab[a];

/*------------------Decode steim data, determine # samples-------------------*/

	oid = 0;
	oarray = &detector->indatar[detector->lbksize];

	l_data = LocGM68_LONG(recordheader->samp_1);
	*oarray++ = l_data;
	oid++;			/* Get first point from forward 
				   integrating constant */

	fskip = TRUE;		/* Skip first delta */

	for (frame=0; frame<16; frame++) {
		for (chunk=0; chunk<15; chunk++) {

   	    		frtype = CtlWordType(LocGM68_LONG(
			    steimframe->framestore[frame].ctlflags),chunk);

			switch (frtype) {
			case 1:
				tdata = steimframe->framestore[frame].
				   chunkstore[chunk].type_1.byteval[0];
				if (fskip) {
					fskip=FALSE;	/* Skip first point */
				} else {
					l_data += tdata;
					*oarray++ = l_data;
					oid++;
				}
				l_data += steimframe->framestore[frame].
				   chunkstore[chunk].type_1.byteval[1];
				*oarray++ = l_data;
				oid++;
				l_data += steimframe->framestore[frame].
				   chunkstore[chunk].type_1.byteval[2];
				*oarray++ = l_data;
				oid++;
				l_data += steimframe->framestore[frame].
				   chunkstore[chunk].type_1.byteval[3];
				*oarray++ = l_data;
				oid++;
				break;
			case 2:
				tdata = LocGM68_WORD(steimframe->framestore[frame].
				   chunkstore[chunk].type_2.wordval[0]);
				if (fskip) {
					fskip=FALSE;	/* Skip first point */
				} else {
					l_data += tdata;
					*oarray++ = l_data;
					oid++;
				}
				l_data += LocGM68_WORD(steimframe->framestore[frame].
				   chunkstore[chunk].type_2.wordval[1]);
				*oarray++ = l_data;
				oid++;
				break;
			case 3:
				tdata = LocGM68_LONG(steimframe->framestore[frame].
				   chunkstore[chunk].type_3.longval);
				if (fskip) {
					fskip=FALSE;	/* Skip first point */
				} else {
					l_data += tdata;
					*oarray++ = l_data;
					oid++;
				}
			}
		}
	}

	detector->datapts = oid;		/* Save count */
}
